/*
	openisis - an open implementation of the ISIS database
	Version 0.8.x (microversion see file Version)
	Copyright (C) 2001-2002 by Erik Grziwotz, erik@openisis.org

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	This software is dedicated to the memory of Eckart Dietrich.

	This software is inspired by (but contains no code of) the iAPI
	Copyright (C) 2000 by Robert Janusz, rj@jezuici.krakow.pl.
	See iAPI.txt for what it contains.
EOH */


package org.openisis;

import java.util.StringTokenizer; // split


/**
	Field implementation of openisis java binding.
	A Field is an immutable structure, binding a String value to a tag.
	It provides methods for formatted access to the value.
<p>
	$Id: Field.java,v 1.1 2002/05/08 00:43:04 kripke Exp $
	@version $Revision: 1.1 $
	@author  $Author: kripke $
*/
public class Field {


	/**
		what a single field can contribute to the V operator.
		@param b a StringBuffer to append to
		@param mode OR-combination of the field formatting flags
			defined in Db and an optional subfield character.
		@param plain a String to format
		@param len a 16bit length constraint | offset &lt;&lt; 16
		@return false iff a non-existing subfield was requested, else true
	*/
	public static boolean v ( StringBuffer b, int mode, String plain, int len ) {
		if ( 0 != (Db.MXU & mode) ) // won't change any specials
			plain = plain.toUpperCase();
		final char[] c = plain.toCharArray();
		final int cl=c.length;
		char sub = (char)mode;
		int o=0, e=cl;
		if ( 0 != sub ) {
			if ( '*' == sub ) { // first subfield
				if ( e > 1 && '^' == c[0] )
					o = 2;
			} else {
				if ( 0 != (Db.MXU & mode) ) // oops - upcased subfield marks ...
					sub = Character.toUpperCase(sub);
				for (;;) {
					if ( cl == o ) return false; // no subfield
					if ( '^' != c[o++] ) continue;
					if ( cl == o ) return false; // no subfield
					if ( sub == c[o++] ) break;
				}
			}
			// got sub starting at o
			e = o;
			while ( cl > e && '^' != c[e] )
				e++;
		}
		// now the relevant data is at offset o and ends at e
		if ( 0 != len ) {
			int off = len >>> 16;
			len |= 0xffff;
			o += off;
			if ( 0 != len && e > o+len )
				e = o+len;
		}
		if ( e <= o )
			return true; // existing but empty
		boolean mh = Db.MHL == (Db.MHL & mode);
		boolean mi = Db.MI  == (Db.MI  & mode);
		boolean ht = Db.HTU == (Db.HTU & mode);
		boolean ab = false; // inside angel brackets: stop at =
		if ( ! (mh || ht) ) {
			b.append( c, o, e-o );
			return true;
		}
		final char htlim = // chars > htlim are escaped
			( Db.HTA == (Db.HTA & mode) ) ? 127 :
			( Db.HTI == (Db.HTI & mode) ) ? 255 :
			Character.MAX_VALUE;
		int i=o;
		char x = 0;
		// now we run at least one of mh or ht mode replacements
		LOOP: for (;;i++) {
			if ( i < e && htlim >= (x = c[i]) )
				switch ( x ) {
				case '"': case '&': if ( ht ) break; continue;
				case '^':           if ( mh ) break; continue;
				case '=':           if ( ab ) break; continue;
				case '<': case '>': break; // special in any case
				default:            continue;
				}
			if ( i > o ) // flush what we had up to current position
				b.append( c, o, i-o );
			if ( i == e ) // done
				break;
			o = i+1; // new offset after this
			if ( htlim < x ) { // write decimal value
				b.append( "&#" ).append( (int)x ).append( ';' );
				continue;
			}
			// now we have one of the special chars
			switch ( x ) {
			case '"': b.append( "&quot;" ); continue; // ht
			case '&': b.append( "&amp;" ); continue; // ht
			case '^': // mh
				if ( i+1 < e ) {
					x = Character.toUpperCase( c[++i] );
					if ( 1 < o )	// not on beginning of field
						b.append( ('A' == x) ? ';' : ('A'<x && x<'J') ? ',' : '.' )
							.append( ' ' );
				}
				o = i+1;
				continue;
			case '=': // angel brackets -- skip to >
				while ( '>' != c[i] ) if ( e == ++i ) break LOOP; // ran to end
				o = i+1;
				// don't break, go on checking for ><-pair
			case '>':
				ab = false;
				if ( ! mh )
					b.append( "&gt;" );
				else if ( i+1<e && '<' == c[i+1] )
					b.append( "; " );
				continue;
			case '<':
				if ( ! mh ) { b.append( "&lt;" ); continue; } // ht
				if ( ! mi ) { ab = true; continue; } // non-index mh
				// index mode -- skip to = or >
				while ( '>' != c[i] && '=' != c[i] )
					if ( e == ++i ) break LOOP; // ran to end
				o = i+1;
				break;
			}
		}

		if ( Db.MDL == (Db.MDL & mode) ) // data mode
			switch ( Character.getType( c[e-1] ) ) {
			case Character.START_PUNCTUATION:
			case Character.END_PUNCTUATION:
			case Character.CONNECTOR_PUNCTUATION:
			case Character.OTHER_PUNCTUATION:
				b.append( "  " );
				break;
			default:
				b.append( ".  " );
			}

		return true;
	}	// v


	/** format a String in HTU-mode.
		@see #v(StringBuffer,int,String,int)
	*/
	public static String html ( String plain ) {
		StringBuffer b = new StringBuffer( plain.length() + 3 );
		v( b, Db.HTU, plain, 0 ); // always true w/o subfield spec
		return b.toString();
	}	// html


	//
	// so far the statics
	//

	/** the tag of this field.
	For primary fields, this is the field number.
	For subfields, this is actually a char.
	*/
	public final int tag;
	public final String val;
	
	public Field ( int tag_, String val_ ) {
		tag = tag_;
		val = val_;
	}

	/** split the Field into subfields.
	*/
	public Field[] split () {
		if ( null == val || 0 == val.length() )
			return null;
		StringTokenizer st = new StringTokenizer( val, "^" );
		Field[] f = new Field[ st.countTokens() ];
		for ( int i=0; i<f.length; i++ ) {
			String tok = st.nextToken();
			if ( null != tok && 0 < tok.length() )
				f[i] = new Field( (int)tok.charAt(0), tok.substring(1) );
		}
		return f;
	}	// split


	/** format field to a StringBuffer.
		@see #v(StringBuffer,int,String,int)
	*/
	public boolean v ( StringBuffer b, int mode ) {
		return v( b, mode, val, 0 );
	}	// v


	/** format field to a StringBuffer.
		@see #v(StringBuffer,int,String,int)
	*/
	public boolean v ( StringBuffer b, int mode, int len ) {
		return v( b, mode, val, len );
	}	// v


	/** format field as new String.
		@see #v(StringBuffer,int,String,int)
	*/
	public String v ( int mode, int len ) {
		StringBuffer b = new StringBuffer( val.length() + 3 );
		return v( b, mode, val, len ) ? b.toString() : Db.EMPTY;
	}	// v


	/** format field as new String.
		@see #v(StringBuffer,int,String,int)
	*/
	public String v ( int mode ) {
		return v( mode, 0 );
	}	// v


	/** format field in HTU-mode as new String.
		@see #v(StringBuffer,int,String,int)
	*/
	public String html () {
		return v( Db.HTU );
	}	// html

}	// Field
