/*
	openisis - an open implementation of the ISIS database
	Version 0.8.x (microversion see file Version)
	Copyright (C) 2001-2002 by Erik Grziwotz, erik@openisis.org

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	This software is dedicated to the memory of Eckart Dietrich.

	This software is inspired by (but contains no code of) the iAPI
	Copyright (C) 2000 by Robert Janusz, rj@jezuici.krakow.pl.
	See iAPI.txt for what it contains.
EOH */

package org.openisis;

/**
	Record implementation of openisis java binding.
<p>
	$Id: Rec.java,v 1.1 2002/05/08 00:43:04 kripke Exp $
	@version $Revision: 1.1 $
	@author  $Author: kripke $
*/
public class Rec {

	private int rowid;
	private Field[] fields;
	
	public Rec ( int rowid_, Field[] fields_ ) {
		rowid = rowid_;
		fields = fields_;
	}

	public int getRowId () { return rowid; }

	public int getLen () { return fields.length; }

	public Field getField ( int i ) { return fields[i]; }

	/** the field selector.
		format one or more occurences of a field to a StringBuffer,
		mimicking the ISIS Vn-operator.
		@param b the StringBuffer to fill
		@param cpre conditional prefix: printed first, if anything is printed
		@param rpre repeated prefix: printed before every occ
			(but not the first, if mode include PP)
		@param tag the field number (0 = all)
		@param mode several mode bits as defined in Db + optional subfield char
		@param occ a 16bit occ | end &lt;&lt; 16.
			if occ is 0, all occurences are printed.
			if the end (higher bytes) are 0, only this occ is printed,
			else all up to incl. end (use Db.LOCC for all).
		@param len a 16bit length constraint | offset &lt;&lt; 16
			this is passed to Field.v and applied for each occ.
		@param rsuf repeated suffix: printed after every occ
			(but not the last, if mode include PS)
		@param csuf conditional suffix: printed last, if anything was printed
		@return the number of formatted occurences
		@see Field#v(StringBuffer,int,String,int)
	*/
	public int v ( StringBuffer b,
		String cpre, String rpre,
		int tag, int mode, int occ, int len,
		String rsuf, String csuf )
	{
		int end = occ >>> 16; occ &= 0xffff;
		int o = 0; // occurences seen
		int did = 0; // occurences done (after range and subfield selection)
		int lrsuf = -1; // last pos of repeated suffix

		// data mode ending should not be used with suffix ?
		// if ( null != rsuf || null != csuf ) mode &= ~Db.MD;

		// loop all fields
		for ( int i=0; i<fields.length; i++ ) {
			if ( 0 != tag && fields[i].tag != tag )
				continue;
			o++; // this is the oth occ, counting from 1
			// System.err.println( "tag: "+tag+" occ: "+o );
			if ( 0 != occ ) // occ range limit
				if ( o < occ )
					continue;
				else if ( o > occ && o > end )
					break;
			// System.err.println( "in range");
			// seems we are in range
			int l = b.length();
			if ( 0 == did && null != cpre )
				b.append( cpre );
			if ( null != rpre && (0<did	|| 0==(mode&Db.PP)) )
				b.append( rpre );
			boolean ok;
			if ( 0 == ((Db.DS|Db.NS) & mode) ) // no dummy
				ok = fields[i].v( b, mode, len ); // really print it
			else if ( 0 == (mode & 0xffff) || '*' == (mode & 0xffff) )
				ok = true; // no or 1st subfield
			else {
				char[] sf = new char[2];
				sf[0] = '^'; sf[1] = (char)(mode & 0xffff);
				ok = 0 > fields[i].val.indexOf( new String(sf) );
				if ( 0 != (Db.NS & mode) ) // the N absence dummy
					ok = !ok;
			}
			if ( ! ok ) {
				// subfield not found -- however, it counts as an field occ ...
				// backup to where we were
				b.delete( l, Integer.MAX_VALUE );
				continue;
			}
			did++; // gotcha
			if ( null == rsuf )
				continue;
			lrsuf = b.length();
			b.append( rsuf );
		}
		if ( 0 <= lrsuf && 0 != (Db.PS & mode) ) // backup over lrsuf
			b.delete( lrsuf, Integer.MAX_VALUE );
		if ( 0 != did && null != csuf )
			b.append( csuf );
		return did;
	}	// v

	/** format fields to a new StringBuffer, return it's content as String.
		@return a new String or the String Db.EMPTY, iff no fields where done.
		@see #v(StringBuffer,String,String,int,int,int,int,String,String)
	*/
	public String v ( String cpre, String rpre,
		int tag, int mode, int occ, int len,
		String rsuf, String csuf )
	{
		StringBuffer b = new StringBuffer();
		return 0 < v( b, cpre, rpre, tag, mode, occ, len, rsuf, csuf )
			? b.toString() : Db.EMPTY;
	}	// v


	/** format fields to a new StringBuffer, return it's content as String.
		the conditional pre and suf are null here.
		@return a new String or the String Db.EMPTY, iff no fields where done.
		@see #v(StringBuffer,String,String,int,int,int,int,String,String)
	*/
	public String v (
		String rpre, int tag, int mode, int occ, int len, String rsuf )
	{
		return v( null, rpre, tag, mode, occ, len, rsuf, null );
	}	// v

}	// Rec
