#
#/*
#	openisis - an open implementation of the ISIS database
#	Version 0.8.x (microversion see file Version)
#	Copyright (C) 2001-2002 by Erik Grziwotz, erik@openisis.org
#
#	This library is free software; you can redistribute it and/or
#	modify it under the terms of the GNU Lesser General Public
#	License as published by the Free Software Foundation; either
#	version 2.1 of the License, or (at your option) any later version.
#
#	This library is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#	Lesser General Public License for more details.
#
#	You should have received a copy of the GNU Lesser General Public
#	License along with this library; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#	This software is dedicated to the memory of Eckart Dietrich.
#
#	This software is inspired by (but contains no code of) the iAPI
#	Copyright (C) 2000 by Robert Janusz, rj@jezuici.krakow.pl.
#	See iAPI.txt for what it contains.
#EOH */
#
#	$Id: OpenIsis.pm,v 1.10 2002/06/24 11:22:26 kripke Exp $
#	perl module wrapper for openisis perl XSUB
#
package OpenIsis;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);
$VERSION = '0.852';

bootstrap OpenIsis $VERSION;

# query mode constants
*QRY_KEYEQ = \0;
*QRY_KEYPF = \1;
*QRY_KEYAT = \2;
*QRY_SCANE = \64;
*QRY_SCANC = \65;
*QRY_SIMPLE = \128;
*QRY_PROPER = \129;

# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for the OpenIsis module.

=head1 NAME

OpenIsis - Perl extension for accessing ISIS databases

=head1 SYNOPSIS

  use OpenIsis;
  $db = OpenIsis::open( 'basename', '-dbpath', '/home/me/mydb' );
	$maxmfn = OpenIsis::maxRowid( $db );
	for (OpenIsis::query( $db, 'plant * water' )) {
		my $row = OpenIsis::read( $db, $_ ); # get hashref
		$row = OpenIsis::read( $db, $_, 'MHL V24,V26,V71' ); # read formatted
		my $v24 = $row->{'24'}; # field tags map to arrayrefs
		OpenIsis::MHL( @$v24 ); # apply heading mode to array
		print $row->{'mfn'}, "TITLE :", join("\n   ", @$v24 ), "\n";
		my $v30 = $row->{'30'}->[0]; # contents is like ^ap. 233-238 ^billus.
		my $subfields = OpenIsis::subfields( $v30 ); # get subfield values
		print "a = ", $subfields->{'a'}, " b = ", $subfields->{'b'}, "\n";
	}
	# print terms starting with A
	print join('; ',OpenIsis::terms( $db, 'a$' )), "\n";

=head1 DESCRIPTION

The OpenIsis perl extension provides access to ISIS databases
based on the openisis C library.


=head1 AUTHOR

	Erik Grziwotz, erik@openisis.org

=head1 SEE ALSO

perl(1).

=cut

sub MHL {
	for ( @_ ) {
		s/(<[^=>]*)=[^>]+>/\1>/g; # dump <a=b> substitutions
		s/></; /g; # replace >< pairs
		s/[><]//g; # nuke other ><
		s/^\^.//; # kill initial subfield spec
		s/\^a/; /; # ^a -> ;
		s/\^[b-i]/, /g; # ^[b-i] -> ,
		s/\^./. /g; # others -> .
	}
}
